/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Red Hat, Inc.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Kai Engert <kaie@redhat.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"

interface nsIInterfaceRequestor;
interface nsIX509Cert;
interface nsISSLStatus;

[scriptable, uuid(0729ce8e-8935-4989-ba72-a2d6307f2365)]
interface nsISSLCertErrorDialog : nsISupports
{
  /**
   *  Called when an SSL connection aborts because of a bad certificate,
   *  and no other code is in place for reporting the problem.
   *  Should bring up a dialog to inform the user and display the certificate.
   *
   *  @param status Might be used to query additional information
   *  @param cert The certificate that this error is about
   *  @param textErrorMessage An error message with whitespace formatting
   *  @param htmlErrorMessage Optional, might either be empty,
                              or might contain the same message as in
                              textErrorMessage plus some html formatting.
   *  @param hostName The error occurred when connecting to this hostName.
   *  @param portNumber The error occurred when connecting to this portNumber.
   */
  void showCertError(in nsIInterfaceRequestor ctx, 
                     in nsISSLStatus status,
                     in nsIX509Cert cert,
                     in AString textErrorMessage,
                     in AString htmlErrorMessage,
                     in ACString hostName,
                     in PRUint32 portNumber);
};

%{C++
#define NS_SSLCERTERRORDIALOG_CONTRACTID "@mozilla.org/nsSSLCertErrorDialog;1"
%}
